@@ -57,6 +57,9 @@ class UartManager(threading.Thread):
                 time.sleep(0.1)
             self.reset()
 
+            self.ready = False
+            self.initialize()
+
     def reset(self):
         if self.running:
             self._attemptingIndex = 0
@@ -64,46 +67,47 @@ class UartManager(threading.Thread):
             self._uartBridge = None
             self.port = None
 
-            self.initialize()
-
     def echo(self, string):
         controlPacket = ControlPacket(ControlType.UART_MESSAGE).loadString(string).getPacket()
         uartPacket    = UartWrapper(UartTxType.CONTROL, controlPacket).getPacket()
         UartEventBus.emit(SystemTopics.uartWriteData, uartPacket)
 
     def initialize(self):
-        if not self.running:
-            return
+        while self.running and not self.ready:
+            self.ready = False
+            _LOGGER.debug(F"Initializing... {self.port} {self.baudRate}")
 
-        self.ready = False
-        _LOGGER.debug(F"Initializing... {self.port} {self.baudRate}")
-        if self.port is not None:
-            found_port = False
-            index = 0
-            # check if the provided port is among the listed ports.
-            for testPort in self._availablePorts:
-                if self.port == testPort.device:
-                    found_port = True
-                    self._attemptingIndex = index
-                    break
-                index += 1
-            # if it is not in the list of available ports
-            if not found_port:
-                _LOGGER.debug(F"Setup connection to...{self.port} {self.baudRate}")
-                self.setupConnection(self.port)
-            else:
-                _LOGGER.debug(F"Could not find provided port, attempt connection to index...{self._attemptingIndex} {self.baudRate}")
-                self._attemptConnection(self._attemptingIndex, False)
-            return
+            # if the user provides his own port, we check if it is in the list of ports we have
+            # if it exists, we start there and skip the handshake.
+            # if it is not in the list, we attempt the connection regardless, this might change in the future.
+            if self.port is not None:
+                found_port = False
+                index = 0
+                # check if the provided port is among the listed ports.
+                for testPort in self._availablePorts:
+                    if self.port == testPort.device:
+                        found_port = True
+                        self._attemptingIndex = index
+                        break
+                    index += 1
+                # if it is not in the list of available ports
+                if not found_port:
+                    # we will attempt it anyway
+                    _LOGGER.debug(F"Could not find provided port, attempt connection to index...{self._attemptingIndex} {self.baudRate}")
+                    self.setupConnection(self.port)
+                else:
+                    _LOGGER.debug(F"Setup connection to...{self.port} {self.baudRate}")
+                    self._attemptConnection(self._attemptingIndex, False)
+                continue
 
 
-        if self.port is None:
-            if self._attemptingIndex >= len(self._availablePorts): # this also catches len(self._availablePorts) == 0
-                _LOGGER.debug("No Crownstone USB connected? Retrying...")
-                time.sleep(1)
-                self.reset()
-            else:
-                self._attemptConnection(self._attemptingIndex)
+            if self.port is None:
+                if self._attemptingIndex >= len(self._availablePorts): # this also catches len(self._availablePorts) == 0
+                    _LOGGER.debug("No Crownstone USB connected? Retrying...")
+                    time.sleep(1)
+                    self.reset()
+                else:
+                    self._attemptConnection(self._attemptingIndex)
 
 
     def _attemptConnection(self, index, handshake=True):
@@ -145,8 +149,6 @@ class UartManager(threading.Thread):
             self._uartBridge.stop()
             while self._uartBridge.started and self.running:
                 time.sleep(0.1)
-
-            self.initialize()
         else:
             _LOGGER.info("Connection established to {}".format(port))
             self.port = port
